# ====================================================================
# Copyright (c) 1999 GlobeCom GCAB AB.  All rights reserved.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions
# are met:
#
# 1. Redistributions of source code must retain the above copyright
#    notice, this list of conditions and the following disclaimer. 
#
# 2. Redistributions in binary form must reproduce the above copyright
#    notice, this list of conditions and the following disclaimer in
#    the documentation and/or other materials provided with the
#    distribution.
#
# 3. All advertising materials mentioning features or use of this
#    software must display the following acknowledgment:
#    "This product includes software developed by GlobeCom GCAB AB
#    <http://www.globecom.net/> for use in GlobeCom Jukebox."
#
# 4. The names "GlobeCom Jukebox" and "GlobeCom" must not be used to
#    endorse or promote products derived from this software without
#    prior written permission. For written permission, please contact
#    hpj@globecom.net.
#
# 5. Products derived from this software may not be called "GlobeCom
#    Jukebox" nor may "GlobeCom" appear in their names without prior written
#    permission of the GlobeCom GCAB AB.
#
# 6. Redistributions of any form whatsoever must retain the following
#    acknowledgment:
#    "This product includes software developed by GlobeCom GCAB AB
#    <http://www.globecom.net/> for use in GlobeCom Jukebox."
#
# THIS SOFTWARE IS PROVIDED BY GLOBECOM GCAB AB ``AS IS'' AND ANY
# EXPRESSED OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
# IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
# PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL GLOBECOM GCAB AB OR
# ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
# SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
# NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
# LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
# HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
# STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
# ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED
# OF THE POSSIBILITY OF SUCH DAMAGE.
# ====================================================================

use DBI;
use MPEG::MP3Info;
use strict;

my $dbh=DBI->connect("dbi:mysql:database=$DBNAME",$DBUSER,$DBPASS) || die;

my $exitfail=1;
my $all;
my @id;
my $file;
if ($ARGV[0] eq "-all") {
    $all=1;
    my $res=$dbh->prepare("select id from songs where format = 'mp3'");
    $res->execute();
    while(my ($id)=$res->fetchrow_array) {
	push(@id,$id);
    }
} else {
    my $id;
    ($id,$file)=@ARGV;
    if (@ARGV==3) {
	my $res=$dbh->prepare("select songid from ripqueue where id = $id");
	$res->execute();
	$exitfail=0;
	($id)=$res->fetchrow_array;
	undef $res;
    }
    push(@id,$id);
}

#close STDERR; # We don't wan't any errors screwing with filenames

exit $exitfail
    if (!$MPG123);

my $upd=$dbh->prepare("update songs set difvol = ?,info = ?,length=?,size=? where id = ?");
for my $id (@id) {
    if (!$file||$all) {
	my $res=$dbh->prepare("select file from songs where id = $id");
	$res->execute();
	($file)=$res->fetchrow_array;
    }

    my $mp3info=get_mp3info($file);
    my $info;
    my $length;
    my $size;

    if ($mp3info) {
	my %mp3info=%$mp3info;

	my ($dev,$ino,$mode,$nlink,$uid,$gid,$rdev,$filesize,
	    $atime,$mtime,$ctime,$blksize,$blocks)
	    = stat($file);
	$size=$filesize;

	$info="Bitrate:".$mp3info{BITRATE}."\n";
	$info.="Format: ".$mp3info{FREQUENCY}."Khz, MPEG ".$mp3info{VERSION}." Layer ".$mp3info{LAYER}."\n";
	if ($mp3info{MODE}==0) {
	    $info.="Mode: Stereo\n";
	} elsif ($mp3info{MODE}==1) {
	    $info.="Mode: Joint-Stereo\n";
	} elsif ($mp3info{MODE}==2) {
	    $info.="Mode: Dual Channel\n";
	} elsif ($mp3info{MODE}==3) {
	    $info.="Mode: Mono\n";
	} else {
	    $info.="Mode: ".$mp3info{MODE}."\n";
	}
	if ($mp3info{VBR}) {
	    $info.="Variable bitrate\n";
	}

	$length=$mp3info{MM}*60+$mp3info{SS};
    }

    {
	my %info;
	my $res=$dbh->prepare("select artist,title,volume,trackno from songs where id = $id");
	$res->execute();
	my ($artist,$title,$volume,$trackno)=$res->fetchrow_array;
	if ($artist) {
	    $info{ARTIST}=$artist;
	}
	if ($title) {
	    $info{TITLE}=$title;
	}
	if ($volume) {
	    $info{ALBUM}=$volume;
	}
	if ($trackno) {
	    $info{TRACKNUM}=$trackno;
	}
	$info{YEAR}="";
	$info{COMMENT}="";
	$info{GENRE}="";

	if (defined %info&&-w $file) {
	    set_mp3tag($file,\%info);
	}
    }

    my $pid;
    pipe PIN,POUT;
    if (!($pid=fork)) {
        undef $upd;
        $dbh->disconnect;
	open(STDOUT,">&POUT");
	open (PIN,"-|") || do {
	    close STDERR;
	    exec $MPG123,"-n","1000","-s","-m",$file;
	    exit(1);
	};
	open (POUT,"|$JUKEBOXBIN/analyse");

	my $var;
	while (sysread(PIN,$var,1024)) {
	    syswrite(POUT,$var,length($var));
	}

	close PIN;
	exit 1
	    if ($?>>8);
	close POUT;
	exit 0;
    }
    waitpid($pid,0);

    if ($?>>8) {
	if ($all) {
	    print "Skipped $file\n";
	} else {
	    exit $exitfail;
	}
    }

    $_=<PIN>;
    chomp;
    my $difvol=5-$_;

    $upd->execute($difvol,$info,$length,$size,$id);

    print "Set $file to $difvol and length $length\n"
	if ($all);
}

undef $upd;
$dbh->disconnect;
print "\n";
exit 0;
