# ====================================================================
# Copyright (c) 1999 GlobeCom GCAB AB.  All rights reserved.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions
# are met:
#
# 1. Redistributions of source code must retain the above copyright
#    notice, this list of conditions and the following disclaimer. 
#
# 2. Redistributions in binary form must reproduce the above copyright
#    notice, this list of conditions and the following disclaimer in
#    the documentation and/or other materials provided with the
#    distribution.
#
# 3. All advertising materials mentioning features or use of this
#    software must display the following acknowledgment:
#    "This product includes software developed by GlobeCom GCAB AB
#    <http://www.globecom.net/> for use in GlobeCom Jukebox."
#
# 4. The names "GlobeCom Jukebox" and "GlobeCom" must not be used to
#    endorse or promote products derived from this software without
#    prior written permission. For written permission, please contact
#    hpj@globecom.net.
#
# 5. Products derived from this software may not be called "GlobeCom
#    Jukebox" nor may "GlobeCom" appear in their names without prior written
#    permission of the GlobeCom GCAB AB.
#
# 6. Redistributions of any form whatsoever must retain the following
#    acknowledgment:
#    "This product includes software developed by GlobeCom GCAB AB
#    <http://www.globecom.net/> for use in GlobeCom Jukebox."
#
# THIS SOFTWARE IS PROVIDED BY GLOBECOM GCAB AB ``AS IS'' AND ANY
# EXPRESSED OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
# IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
# PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL GLOBECOM GCAB AB OR
# ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
# SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
# NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
# LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
# HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
# STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
# ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED
# OF THE POSSIBILITY OF SUCH DAMAGE.
# ====================================================================

use strict;
use File::Copy;
use CPAN;
use Sys::Hostname;
use Cwd;

my $curdir = getcwd();

my $batch=0;
my $first = 1;

if ($ARGV[0] eq "auto") {
    print "Installing from cached or default settings\n";
    shift(@ARGV);
    $batch=1;
}

if (@ARGV) {
    print "Usage:\n\n  build [auto]\n\n";
    exit 1;
}

my %names=("PARANOIA" => "cdparanoia",
	   "PLAY" => "tplay/esdplay/play",
	   "ENCODER" => "xingmp3enc/lame/bladeenc",
	   "MKDIR" => "mkdir",
	   "MV" => "mv",
	   "CP" => "cp",
	   "PERL" => "perl",
	   "NICE" => "nice",
	   "AUMIX" => "aumix",
	   "PIDOF" => "pidof",
	   "ECHO" => "echo",
	   "MPG123" => "mpg123",
	   "MYSQL" => "mysql",
	   "MYSQLADMIN" => "mysqladmin",
	   "ATQ" => "atq",
	   "ATRM" => "atrm",
	   "AT" => "at",
	   "WGET" => "wget",
	   "-APM" => "apm",
	   "-MPMF20" => "mpmf20",
	   "-RIO" => "rio"
	   );

my @scripts=(
	  "*analyse",            # Binaries
	  "*globecomice",
	  "*playcd",             # Binaries
	  "-jukebox",            # Shell prog
	  "-jukebox.vanilla",    # Shell prog
	  "addmp3id3.pl",
	  "createalbums.pl",
	  "difvol.pl",
	  "download.pl",
	  "encode.pl",
	  "getnextmp3.pl",
	  "movevol.pl",
	  "mpmf20.pl",
	  "organizemp3.pl",
	  "playit.pl",
	  "playmp3.pl",
	  "playwav.pl",
	  "remove.pl",
	  "removegroup.pl",
	  "ripd.pl",
	  "ripping.pl",
	  "setmp3id3.pl",
	  "setvol",
	  "trackno.pl",
	  "upload.pl"
	  );

my @perlMod=("DBI",
	     "DBD::mysql",
	     "MIME::Base64",
	     "MPEG::MP3Info");

my $PERL;
my %askcache;

sub confirmCPAN { # JBM - check CPAN and ask the user if they want to use it.
    my ($modNam)=$_[0];
    my $ret;

print <<__EOT__;
==================================================================
Perl module \'$modNam\' is needed by Jukebox and is not installed
The CPAN module can automatically download, build and install it.

If you have not used CPAN before you will be asked to configure it. It takes a few
minutes and involves some automated downloads. It is worth doing this because CPAN
makes life much easier. You can accept the defaults for most questions except
\'policy on building prerequisites\' where you must answer \'ask\' ( Otherwise
it may decide you need a new version of Perl and you will tear your hair out )

If you prefer to install Perl modules by hand you should quit this installer and
download \'$modNam\'. Untar it into your src directory. cd into the new directory
created and do \'perl Makefile.pl\' \'make\' \'make install\'

__EOT__

    print "Proceed with automatic install via CPAN [y/n]";
    $ret=<STDIN>;
    chomp($ret);
    exit 1 if ($ret =~ m/n/i);
}

sub installMod {
    my ($modNam)=$_[0];

    eval "use $modNam;";
    if ($@) {
	if($first) {
	    &confirmCPAN;
	    $first = 0;
	}
	my $obj = CPAN::Shell->expand('Module',$modNam);
	$obj->install;
	die  "Couldn't install $modNam" if (!$obj->inst_version || $obj->inst_version eq "undef");
    }
}

sub findname {
    my $name=shift(@_);
    for my $cname (split(/\//,$name)) {
	for my $c (split(/:/,$ENV{"PATH"})) {
	    return $c."/".$cname
		if (-x $c."/".$cname);
	}
    }
    my $path;
    do {
	if ($batch) {
	    $path=$askcache{$name};
	} else {
	    print "Can not find program $name enter the path to it or ENTER if you don't have it\n";
	    $path=<STDIN>;
	    chomp($path);
	}
	return
	    if (!length($path));
	if (!-f $path) {
	    $path=~s/\/$//;
	    $path.="/".$name;
	}
    } while ((!-x $path || ! -f $path)&&!$batch);

    return $path;
}

if (open (ASKCACHE,"<.conf.cache")) {
    while(<ASKCACHE>) {
	chomp;
	my ($tag,$val)=(/^([^=]+)=(.*)$/);
	$askcache{$tag}=$val;
    }
    close ASKCACHE;
}

sub ask {
    my ($prompt,$defname,$default,$emptyok)=@_;
    my $ret;

    if (exists $askcache{$defname}) {
	$default=$askcache{$defname};
    }

    if (!$batch) {
	do {
	    print $prompt;
	    
	    if ($default) {
		print "Press enter for [$default] ";
	    }
	    if ($emptyok) {
		print "To specify an empty value use \"\".";
	    }

	    $ret=<STDIN>;
	    chomp($ret);
	    if (!$ret) {
		$ret=$default;
	    }
	} while (!$ret);
    } else {
	$ret=$default;
    }
    if (!$ret) {
	print "Missing value for $defname\n";
	exit 2;
    }

    $askcache{$defname}=$ret;

    if (open (ASKCACHE,">.conf.cache")) {
	for my $key (keys %askcache) {
	    print ASKCACHE "$key=".$askcache{$key}."\n";
	}
	close ASKCACHE;
    }

    if ($ret eq "\"\"") {
	$ret="";
    }

    return $ret;
}

print "Searching for required external programs\n";# changed text here. 'help' programs implies they're for UI help.

my %paths;

for my $prog (keys(%names)) {
    my $name=$names{$prog};
    my $path;
    if ($prog=~s/^-//) {
	if ($path=findname($name)) {
	    $paths{$prog}=$path;
	    print "Found $name at $path\n";
	} else {
	    $paths{$prog}="";
	    print "Skipped $name\n";
	}
	$askcache{$name}=$path;
    } else {
	if ($paths{$prog}=$path=findname($name)) {
	    print "Found $name at $path\n";
	} else {
	    print "Couldn't find $name in $ENV{\"PATH\"}. Jukebox needs this program.\n";
	    exit 1;
	}
    }
}

$PERL=$paths{"PERL"};

print "\nChecking for required PERL modules\n";

for my $mod (@perlMod) {
    &installMod($mod);
}


require DBI; # JBM - moved this here so DBI can be installed above

chdir $curdir; # for some reason I think CPAN stuff changes directory on us


my $MYSQLADMIN=$paths{MYSQLADMIN};
my $MYSQL=$paths{MYSQL};

print "\n";

if (-x "scripts/playcd") {
    my $ret=(system("scripts/playcd")>>8);
    if ($ret!=1) {
	unlink("scripts/playcd");
    }
}
if (-x "scripts/analyse") {
    my $ret=(system("scripts/analyse < /dev/null > /dev/null")>>8);
    if ($ret!=1) {
	unlink("scripts/analyse");
    }
}
if (-x "scripts/globecomice") {
    my $ret=(system("scripts/globecomice < /dev/null > /dev/null")>>8);
    if ($ret!=1) {
	unlink("scripts/globecomice");
    }
}

if (! -x "scripts/playcd") {
    print "Compiling playcd\n";
    if (system ("gcc -o scripts/playcd src/playcd.c -lcdaudio")>>8) {
	exit(1);
    }
}
if (! -x "scripts/analyse") {
    print "Compiling analyse\n";
    if (system ("gcc -o scripts/analyse src/analyse.c")>>8) {
	exit(1);
    }
}
if (! -x "scripts/globecomice") {
    print "Compiling globecomice\n";
    if (system ("gcc -o scripts/globecomice src/globecomice.c -lshout")>>8) {
	print "Icecast will not be supported (You can still use the jukebox via soundcard)\n";
    }
}

print "\nGetting some paths\n";
my $base=&ask(<<__EOT__,"MP3","/mp3", 0);
Enter basepath for MP3 directory. This is where your MP3 files will be placed
by the 'organizemp3.pl' program ( see file SCRIPTS for more info )
If you already have a large collection of MP3 files you should enter the base
directory here 
__EOT__

if (!-d $base) {
    if (system("mkdir","-p",$base)>>8) {
        print "Couldn't create directory $base\n";
        exit (1);
    }
}
print "MP3 files will be stored within $base\n\n";

if (!-d "$base/incoming") {
    my $incoming=&ask(<<__EOT__,"INCOMING","y", 0);
Do you want to create the $base/incoming directory for holding MP3 files while
ripping or uploading ? 
__EOT__
    if ($incoming=~/^y/i) {
	if (system("mkdir","-p","$base/incoming")>>8) {
	    print "Couldn't create directory $base/incoming\n";
	    exit (1);
	}
	my $chown=&ask(<<__EOT__,"CHOWN","y", 0);
Change owner of 'incoming' directory to 'nobody' - needed if you will
upload songs in the Jukebox.
__EOT__
        if ($chown=~/^y/i) {
	    if (system("chown","nobody","$base/incoming")>>8) {
		print "Couldn't chown directory $base/incoming to nobody\n";
		exit (1);
	    }
	}
    }
}

# parse apache config file looking for docroot

open ( APACHECONF, "</etc/httpd/conf/httpd.conf" );

# we're looking for something like : DocumentRoot "/home/httpd/html"

my $docroot = '';
my $guesswebdir = '';

while(<APACHECONF>) {
    chomp;
    if (/^DocumentRoot/) {
	m/\"(.*)\"/;
	$docroot = $1;
	print "Apache DocumentRoot seems to be $docroot\n";
	last;
    }
}
close APACHECONF;

if (!$docroot) {
    $docroot="/var/www/html";
}

$guesswebdir = $docroot."/music/";

my $webbase=&ask(<<__EOT__,"WEBBASE", $guesswebdir, 0);
Enter the directory where the webfiles should be put within Apache DocumentRoot
This directory must have php parsing of .php3 files set up in apache.
__EOT__

if (!-d $webbase) {
    if (system("mkdir","-p",$webbase)>>8) {
        print "Couldn't create directory $webbase\n";
        exit (1);
    }
}

print "Putting webpages at $webbase\n\n";

my $hosturl = "http://".hostname()."/music";

my $weburl=&ask(<<__EOT__,"WEBURL", $hosturl, 0);
Enter the URL to the $webbase directory (Eg http://localhost/music).
__EOT__

print "Webpages are at $weburl\n\n";

my $setvol=&ask(<<__EOT__,"LOCK","/var/lock/setvol.lock", 0);
Enter the lockfile for the volume daemon to use.
__EOT__

my $cdrom=&ask(<<__EOT__,"CDROM","/dev/cdrom", 0);
Enter the device filename for the cdrom, this must be accessible by the user
that the webserver is running as. Press enter for /dev/cdrom.
__EOT__

my $dbname=&ask(<<__EOT__,"DBNAME","music", 0);
Enter the name for the database. Songs, playlists and other stuff
will be stored within this mysql database.
__EOT__

print "Using database $dbname\n\n";

my $dbuser=&ask(<<__EOT__,"DBUSER","web", 0);
Enter the login name for the database user that Jukebox will use to
connect to mysql. This is a mysql user ( NOT Unix user ) and will
be created automatically with the correct priviliges for the $dbname database.
__EOT__

print "Using database user $dbuser\n\n";

my $dbpass=&ask(<<__EOT__,"DBPASS","web", 0);
Enter the login password you would like to assign.
__EOT__

print "Using database user $dbpass\n\n";

my $root=&ask(<<__EOT__,"ROOT","root", 0);
Enter the login name for the database user able to create tables. Again
this is a mysql user, not Unix. A default installation of mysql creates
a user named 'root'. Select that if you don't have a different user
already setup in mysql.
__EOT__

print "Using database admin user $root\n\n";

my $rootpass=&ask(<<__EOT__,"ROOTPASS","\"\"", 1);
Enter the login password for the database user able to create tables.
The default mysql 'root' user has a blank password, so choose that if
you didn't change it in mysql.
 (the characters will be displayed as you type).
__EOT__

print "Using database admin user password $rootpass\n\n";

$rootpass="-p$rootpass"
    if (length($rootpass));

my $binbase=&ask(<<__EOT__,"BINBASE","/usr/local/jukebox", 0);
Enter the directory where the jukebox programs should be put.
__EOT__

if (!-d $binbase) {
    if (system("mkdir","-p",$binbase)>>8) {
        print "Couldn't create directory $binbase\n";
        exit (1);
    }
}
print "Putting programs at $binbase\n\n";

my $othadr=&ask(<<__EOT__,"OTHADR","\"\"", 0);
Adresses other than 127.* addresses that should be allowed to adjust volume,
use * for wildcard (Only numerical addresses). Several adresses can be separated
with a , character.
__EOT__

open (TOP,">scripts/topofperl") || die;
print TOP "#!$PERL\n\nuse strict;\n";

my $tprgstring;

for my $prog (keys(%paths)) {
    my $tprg;
    ($tprg=$prog)=~s/^-//;
    print TOP "my \$$tprg=\"".$paths{$prog}."\";\n";
    $tprgstring = $tprgstring."\$".$tprg."\n";
}

print TOP "my \$SETVOLLOCK=\"$setvol\";\n";
print TOP "my \$DBUSER=\"$dbuser\";\n";
print TOP "my \$DBPASS=\"$dbpass\";\n";
print TOP "my \$DBNAME=\"$dbname\";\n";
print TOP "my \$TARGETBASE=\"$base\";\n";
print TOP "my \$JUKEBOXBIN=\"$binbase\";\n";
print TOP "my \$SITE=\"$webbase\";\n";
print TOP "\$ENV{CDROM}=\"$cdrom\";\n";
print TOP "my \$SITEURL=\"$weburl\";\n";

print TOP 'my @ALLOWED_HOSTS=('."\n".
    "'^127\\..*\$',\n";

for my $line (split(/,/,$othadr)) {
    $line=~s/\./\\./g;
    $line=~s/\*/.*/g;
    print TOP "'^$line\$',\n";
}

print TOP ");\n";
close TOP;

open (TOP,">scripts/topofsh") || die;
print TOP "#!/bin/sh\n\nJUKEBOX=$binbase\nexport JUKEBOX\nAUMIX=".$paths{AUMIX}."\nexport AUMIX\n";
close TOP;

print "Installing binaries to $binbase\n";
for my $prog (@scripts) {
    if ($prog=~s/^-//) {
	system("cat scripts/topofsh scripts/$prog > '$binbase/$prog'");
	system("chmod","a+x","$binbase/$prog");
    } elsif ($prog=~s/^\*//) {
	system("cp","scripts/".$prog,"$binbase/".$prog);
	system("chmod","a+x","$binbase/$prog");
    } else {
	system("cat scripts/topofperl scripts/$prog > '$binbase/$prog'");
	system("chmod","a+x","$binbase/$prog");
    }
}

print "Installing webpages to $webbase\n";
my $at=$paths{AT};
my $atrm=$paths{ATRM};
my $atq=$paths{ATQ};
my $apm=$paths{"APM"};
my $echo=$paths{ECHO};
open (TOP,">www/connect.php3") || die;
print TOP <<__EOPHP__;
<?

set_magic_quotes_runtime(0);
if (get_magic_quotes_gpc()!=0) {
    print "Need to disable magic_quotes_gpc in php.inc or php.ini for Jukebox to work";
}

\$incomingdir="$base/incoming";
\$site="$weburl";
\$webbasedir="$webbase";
\$at="$at";
\$atrm="$atrm";
\$atq="$atq";
\$apm="$apm";
\$binbase="$binbase";
\$echo="$echo";
\$perl="$PERL";

\$MUSICUSER=\$MUSICPRI="";

mysql_pconnect("localhost","$dbuser","$dbpass");
if (!mysql_select_db("$dbname")) {
  print "Can't connect to the database.";
  exit;
}

if (!\$AUTH_USER)
    \$AUTH_USER=\$PHP_AUTH_USER;

if (!\$AUTH_PWD)
    \$AUTH_PWD=\$PHP_AUTH_PW;

if (!\$AUTH_USER)
    \$AUTH_USER="anonymous";

function prepareDB(\$string)
{
	return "'".AddSlashes(\$string)."'";
}

\$authuser=prepareDB(\$AUTH_USER);
\$authpwd=prepareDB(\$AUTH_PWD);

\$result=mysql_query("SELECT pass, queue FROM users WHERE name = \$authuser and pass = password(\$authpwd)");
if (mysql_num_rows(\$result)>0) {
  \$MUSICUSER=\$AUTH_USER;
  list(\$pass,\$MUSICPRI)=mysql_fetch_array(\$result);
}

function helpLink(\$name,\$label="Help")
{
    print "<A HREF=help/\$name.php3 onClick=\\"javascript:window.open('help/\$name.php3','help','scrollbars=yes,toolbar=no,status=no,menubar=no,location=no,height=300,width=550,directories=no,resizable=yes'); return false;\\" onMouseOver=\\"self.status='Help'\\" target=help><font size=-1>\$label</font></A>";
}

function fetchSetting(\$name)
{
	global \$MUSICUSER;
	if (!\$MUSICUSER||!(list(\$val)=mysql_fetch_array(mysql_query("select value from usersetting where name = '\$name' and user = '\$MUSICUSER'"))))
        list(\$val)=mysql_fetch_array(mysql_query("select value from settings where name = '\$name'"));
    return \$val;
}

if (!\$MUSICUSER&&\$REMOTE_ADDR=="127.0.0.1") {
  \$MUSICUSER=fetchSetting("LOCALUSER");
  \$result=mysql_query("SELECT queue FROM users WHERE name = '\$MUSICUSER'");
  list(\$MUSICPRI)=mysql_fetch_array(\$result);
  if (!\$MUSICPRI)
    \$MUSICUSER="";
}

\$onlydown=fetchSetting("ONLYDOWN");

if ((\$onlydown&&\$MUSICPRI>=\$onlydown)||!\$MUSICUSER)
    \$onlydown=1;
else
    \$onlydown=0;

if (\$MUSICUSER&&\$NOTAUTH==\$MUSICUSER) {
    \$MUSICUSER=\$MUSICPRI="";
    \$AUTH=1;
}

\$streamname=fetchSetting("DOWNLOAD");
if (\$streamname) {
    if (\${\$streamname}&&fetchSetting("ANOMSTREAM"))
	streamit(\${\$streamname});
}

if (((\$AUTH||!fetchSetting("ANONYMOUS"))&&!\$MUSICUSER)||(\$AUTH==2&&\$onlydown)) {
  Header("WWW-authenticate: basic realm=\\"Music\\"");
  Header("HTTP/1.0 401 Unauthenticated");
  print "You have no business here anyway!\\n";
  exit;
}

if (!\$theme) {
    \$theme=fetchSetting("THEME");

    require "\$theme/theme.php3";
}

\$CONNECTED=1;

?>
__EOPHP__
close(TOP);
use File::Copy;
copy("NEWS","www/NEWS");
system("(cd www ; tar cpf - *) | ( cd $webbase ; tar xf -)");

print "Creating database music with mysql\n";
my $dbh;
my $newdb;
if (!(system("$MYSQLADMIN -u$root $rootpass create $dbname")>>8)) {
    print "Setting up user $dbuser\n";
    open(SQL,"|$MYSQL -u$root $rootpass mysql -B");
    print SQL <<__EOSQL__;
insert into db (host,db,user,select_priv,insert_priv,update_priv,delete_priv) values ('localhost','$dbname','$dbuser','y','y','y','y');
insert into user (host,user,password) values ('localhost','$dbuser',password('$dbpass'));
__EOSQL__
    close SQL;
    print "Creating tables in $dbname\n";
    system("$MYSQL -u$root $rootpass $dbname -B < sql/tables.sql");
    system("$MYSQLADMIN -u$root $rootpass reload");
    print "Saving settings\n";
    system("$MYSQL -u$root $rootpass $dbname -B < sql/settings.sql");
    $newdb=1;
} else {
    $dbh=DBI->connect("dbi:mysql:database=$dbname",$dbuser,$dbpass) || die "Can't connect to existing database";

    my $res=$dbh->prepare("select value from settings where name = 'VERSION'");
    $res->execute;
    my ($version)=$res->fetchrow_array();
    if (!$version) {
	print "ERROR: This database doesn't seem to be a GlobeCom Jukebox database\n";
	exit 2;
    }
    undef $res;
    $dbh->disconnect;
    print "Database seems to exist, checking for updates for $version\n";
    opendir(SQL,"sql") || die "Can't open sql/ directory";
    my @files;
    while(my $dir=readdir(SQL)) {
	if ($dir=~/^update(\d*).sql$/) {
	    if ($1>=$version) {
		push(@files,$dir);
	    }
	}
    }
    closedir(SQL);
    @files=sort(@files);
    for my $file (@files) {
	print ("Running $file\n");
	system("$MYSQL -u$root $rootpass -B $dbname -f < sql/$file");
    }
}

system("touch $binbase/.cdserverrc");
system("chmod a+rw $binbase/.cdserverrc");

print <<__EOT__;

A service on port 1339 called volume will be added to your inetd / xinetd.

The program providing this service is $binbase/setvol ( written in Perl ) and will 
only accept connects from localhost,  but you might want to block port 1339 
in your firewall if you are paranoid (Read smart:).

Start / stop the Jukebox with \'$binbase/jukebox start|stop\'. This file can
also be placed inside /etc/rc.d/init.d on RedHat systems (And with minimal 
tweaking other systems). If you have problems with this script you should use
$binbase/jukebox.vanilla which should work with any distribution.

There are some utility scripts available that are not used by the jukebox itself
but might come in handy for an average user :

organizemp3.pl     Moves mp3 files from the place where they lie into the MP3
                   directory giving it filenames based on the artist and title
                   in the database. Specifying any options will force moving the
                   database pointing to the new filename even if the move failed.

addmp3id3.pl       Search the specified directories or files recursively for files
                   ending in .mp3 and containing ID3 tags and then adding them into
                   the jukebox based on that information. This program will only be
                   available if you have mp3id3 installed. Very handy for importing
                   huge directories of existing mp3 files into the database. 

setmp3id3.pl       Set the ID3 tags of the songs added in the Jukebox according to
                   the contents of the database content. Specifying any options
                   will remove all current ID3 data. This program will only be
                   available if you have mp3id3 installed.

removegroup.pl     Remove songs belonging to the specified group from the jukebox
                   and from the filesystem.

difvol.pl -all     Set the reference volume of songs entered in the jukebox
                   (Warning, will erase all manual modifications).

createalbums.pl -h Create albums from a database created by addmp3id3.pl or
                   upgraded from Jukebox 3.x

__EOT__

if ($newdb) {
    print <<__EOT__;
An account has been created in the Jukebox with login "admin" and password "2bchanged",
use this account to set up your own account and then delete it!

__EOT__
}

my $bootinetd=0;
my $bootxinetd=0;
my $hasvol=0;

if (open (INETD,"</etc/inetd.conf")) {
    while(<INETD>) {
	if (/^volume/) {
	    $hasvol=1;
	    last;
	}
    }
    close INETD;
    if (!$hasvol) {
	print "Added Jukebox volume service to inetd.conf\n";
	open(INETD,">>/etc/inetd.conf") || die "Couldn't open inet.conf for append";
	print INETD "\n#GlobeCom Jukebox\nvolume stream  tcp     nowait    root      $binbase/setvol\n";
	close INETD;
	$bootinetd=1;
    }
} elsif (-e "/etc/xinetd.conf") {
    if (-d "/etc/xinetd.d" && !-f "/etc/xinetd.d/volume") {
	print "Added Jukebox volume server to /etc/xinetd.d\n";
	copy("scripts/volume","/etc/xinetd.d");
	$bootxinetd=1;
    }
} else {
    print "ERROR: Could not open inetd.conf / xinetd.conf file\n";
    print "You need to install inet or ( preferably ) xinet. Otherwise\n";
    print "the volume setting cannot be controlled from the web UI\n";
}

if (open (SERVICES,"</etc/services")) {
    $hasvol=0;
    while(<SERVICES>) {
	if (/^volume/) {
	    $hasvol=1;
	    last;
	}
    }
    close SERVICES;
    if (!$hasvol) {
	print "Added Jukebox volume service to /etc/services\n";
	open(SERVICES,">>/etc/services") || die "Couldn't open services for append";
	print SERVICES "\n#GlobeCom Jukebox\nvolume 1339/tcp\n";
	close SERVICES;
    }
} else {
    print "ERROR: Could not open services file\n";
}

if ($bootinetd) {
    print "Restarting inetd\n";
    system("kill -HUP `pidof inetd`");
}

if ($bootxinetd) { # JBM - xinetd wants a different signal to reload config files
    print "Restarting xinetd\n";
    system("kill -USR1 `pidof xinetd`");
}

