create table songs (
	id		int not null primary key auto_increment,
	artist		varchar(100) not null,
	title		varchar(100) not null,
	volume		varchar(100) not null,
	file		varchar(200),
	played		int,
	voted		int,
	toplist		double,
	lastplayed	datetime,
	entered		datetime,
	difvol		int,
	length		int,
	flags		int,
	format		varchar(10),
	user		varchar(20),
	trackno		int,
	volid		int,
	info		varchar(255),
	size		int,

	index songartid(artist),
	index songtitid(title),
	index songvolid(volume)
);

create table users (
	name		varchar(20) not null primary key,
	pass		varchar(20),
	queue		int not null
);

create table usersetting (
	user		varchar(20) not null,
	name		varchar(20) not null,
	value		varchar(100),
	primary key (user,name)
);

insert into users values ('anonymous',password(''),1);

create table groups (
	id		int not null primary key auto_increment,
	enable		int not null default 0,
	name		varchar(100),
	pri		int not null default 1,
	category	int,
	size		int
);

insert into groups values (0,0,'All',0,NULL,0);
update groups set id = 0;

create table groupmember (
	groupid		int not null,
	songid		int not null,

	primary key (groupid,songid),
	index grpgrpid (groupid),
	index grpsngid (songid)
);

create table groupsetting (
       groupid		int not null,
       user		varchar(20) not null,
       enable		int not null default 0,
       pri		int not null default 1,

       primary key (groupid,user)
);

insert into groupmember values (0,0);

create table queue (
	queueid		int not null primary key auto_increment,
	pri		int not null,
	id		int not null,
	played		int,
	toplist		double,
	user		varchar(20),

	index queuepriid (pri)
);

create table settings (
	name		varchar(20) not null primary key,
	description	varchar(200) not null,
	type		varchar(10) not null,
	value		varchar(100)
);

create table programs (
	id		int not null auto_increment primary key,
	name		varchar(100) not null,
	cover		varchar(200),		
	artist		varchar(100)
);

create table programmember (
	id		int auto_increment primary key,
	programid	int not null,
	songid		int not null,

	index prgidid(programid),
	index songidid(songid)
);

create table remotehosts (
	id		int not null primary key auto_increment,
	name		varchar(100) not null,
	hostname	varchar(200) not null,
	user		varchar(20),
	pass		varchar(20)
);

create table formats (
	id		varchar(10) primary key,
	description	varchar(100) not null,
	program		varchar(100) not null,
	flags		int not null
);

insert into formats values ('mp3','MPEG Layer 3 using mpg123','playmp3.pl',0);
insert into formats values ('wav','WAV player','playwav.pl',1);
insert into formats values ('cd','CD Player, internal','playcd',0);

create table transformations (
	fromid		varchar(10) not null,
	toid		varchar(10) not null,
	priority	int not null,
	description	varchar(100) not null,
	program		varchar(100) not null,

	primary key (fromid,toid)
);

insert into transformations values ('url','%',10,'Ripping from remote URL','download.pl');
insert into transformations values ('%','del',-20,'Remove song','remove.pl');
insert into transformations values ('cd' ,'wav',-20,'Ripping from CD to WAV','ripping.pl');
insert into transformations values ('wav','mp3',30,'Encoding from WAV to MP3','encode.pl');
insert into transformations values ('mp3','analyse',40,'Analyse recording volume','difvol.pl');
insert into transformations values ('%','upload',-50,'Upload to other jukebox','upload.pl');
insert into transformations values ('vol','vol',-30,'Move from different volumes','movevol.pl');
insert into transformations values ('program','mpmf20',-10,'Upload album to MPMan F20','mpmf20.pl');
insert into transformations values ('program','rio',-10,'Upload album to Rio','mpmf20.pl');

create table ripqueue (
	id		int not null primary key auto_increment,
	songid		int,
	fromid		varchar(10) not null,
	toid		varchar(10) not null,
	artist		varchar(100) not null,
	title		varchar(100) not null,
	album		varchar(100),
	file		varchar(200),
	status		int,
	user		varchar(20),
	trackno		int,
	log		text
);

create table volumes (
        id	        int not null primary key auto_increment,
	flags		int not null,
	name		varchar(40) not null,
	mounted		varchar(255)
);
